// ProgressData.cpp: implementation of the ProgressData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "rootfly.h"
#include "ProgressData.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ProgressData::ProgressData()
{
  Reset();
}

ProgressData::~ProgressData()
{

}

void ProgressData::SetExists(int tube, int window, int session)
{
  if ((tube < 0) || (window < 0) || (session < 0))
    return;

  // ensure list is large enough
  if (m_exist_list.size() <= tube)
  {
    m_exist_list.resize(tube + 1, std::vector< std::vector<bool> >());
  }
  if (m_exist_list[tube].size() <= window)
  {
    m_exist_list[tube].resize(window + 1, std::vector<bool>());
  }
  if (m_exist_list[tube][window].size() <= session)
  {
    m_exist_list[tube][window].resize(session + 1, false);
  }

  if (!m_exist_list[tube][window][session])
  {
    m_exist_count++;
  }
  m_exist_list[tube][window][session] = true;

  if (m_tube_list.size() <= tube)
  {
    m_tube_count++;
    m_tube_list.resize(tube + 1, false);
  }
  if (m_window_list.size() <= window)
  {
    m_window_list.resize(window + 1, false);
  }
  if (m_session_list.size() <= session)
  {
    m_session_list.resize(session + 1, false);
  }

  m_tube_list[tube] = m_window_list[window] = m_session_list[session] = true;
}

bool ProgressData::GetExists(int tube, int window, int session)
{
  if ((tube < 0) || (window < 0) || (session < 0))
  {
    return false;
  }

  if ((m_exist_list.size() <= tube) || 
    (m_exist_list[tube].size() <= window) ||
    (m_exist_list[tube][window].size() <= session))
  {
    return false;
  }

  return m_exist_list[tube][window][session];
}

void ProgressData::SetAnnotated(int tube, int window, int session)
{
  if ((tube < 0) || (window < 0) || (session < 0))
    return;

  // ensure list is large enough
  if (m_annotated_list.size() <= tube)
  {
    m_annotated_list.resize(tube + 1, std::vector< std::vector<bool> >());
  }
  if (m_annotated_list[tube].size() <= window)
  {
    m_annotated_list[tube].resize(window + 1, std::vector<bool>());
  }
  if (m_annotated_list[tube][window].size() <= session)
  {
    m_annotated_list[tube][window].resize(session + 1, false);
  }

  if (!m_annotated_list[tube][window][session])
  {
    m_annotated_count++;
  }
  m_annotated_list[tube][window][session] = true;
}

bool ProgressData::GetAnnotated(int tube, int window, int session)
{
  if ((tube < 0) || (window < 0) || (session < 0))
  {
    return false;
  }

  if ((m_annotated_list.size() <= tube) || 
    (m_annotated_list[tube].size() <= window) ||
    (m_annotated_list[tube][window].size() <= session))
  {
    return false;
  }

  return m_annotated_list[tube][window][session];
}

void ProgressData::SetDone(int tube, int window, int session)
{
  if ((tube < 0) || (window < 0) || (session < 0))
    return;

  // ensure list is large enough
  if (m_done_list.size() <= tube)
  {
    m_done_list.resize(tube + 1, std::vector< std::vector<bool> >());
  }
  if (m_done_list[tube].size() <= window)
  {
    m_done_list[tube].resize(window + 1, std::vector<bool>());
  }
  if (m_done_list[tube][window].size() <= session)
  {
    m_done_list[tube][window].resize(session + 1, false);
  }

  if (!m_done_list[tube][window][session])
  {
    m_done_count++;
  }
  m_done_list[tube][window][session] = true;
}

bool ProgressData::GetDone(int tube, int window, int session)
{
  if ((tube < 0) || (window < 0) || (session < 0))
    return false;

  if ((m_done_list.size() <= tube) || 
    (m_done_list[tube].size() <= window) ||
    (m_done_list[tube][window].size() <= session))
  {
    return false;
  }

  return m_done_list[tube][window][session];
}

/*
// return session index of the last annotated session-image for a given tube-window
int ProgressData::GetLastInWindow(int tube, int window)
{
  if ((m_annotated_list.size() <= tube) || 
    (m_annotated_list[tube].size() <= window))
    return -1;

  int last = -1;

  // iterate over session-images within tube-window; note last session index marked "annotated"
  for (bool *first, *i = first = m_annotated_list[tube][window].begin(), *end = m_annotated_list[tube][window].end(); i < end; i++)
    if (*i)
      last = i - first;

  return last;
}

// return session index of the last annotated window-image for a given tube-session
int ProgressData::GetLastInSession(int tube, int session)
{
  if ((m_annotated_list.size() <= tube))
    return -1;

  int last = -1;

  // iterate over window-images within tube-sessoin; note last window index marked "annotated"
  for (std::vector<bool> *first, *i = first = m_annotated_list[tube].begin(), *end = m_annotated_list[tube].end(); i < end; i++)
    if ((i->size() > session) && (*i)[session])
      last = i - first;

  return last;
}

// return fraction equaling the number of annotated session-images (based on last one found) 
// divided by the number of existing session-images for a given tube-window
float ProgressData::GetFractionOfWindow(int tube, int window)
{
  if ((m_exist_list.size() <= tube) || 
    (m_exist_list[tube].size() <= window))
    return 0;

  int last = -1, total = 0, offset = 0, session;

  // iterate over session-images within tube-window; note last session index marked "annotated"
  for (bool *first, *i = first = m_exist_list[tube][window].begin(), *end = m_exist_list[tube][window].end(); i < end; i++)
    if (*i)
    {
      total++;
      if (GetAnnotated(tube, window, session = i - first))
        last = session - offset;
    }
    else
      offset++;

    return last < 0? 0 : ((float)last + 1) / (float)total;
}

// return fraction equaling the number of annotated window-images (based on last one found) 
// divided by the number of existing window-images for a given tube-session
float ProgressData::GetFractionOfSession(int tube, int session)
{
  if (m_exist_list.size() <= tube)
    return 0;

  int last = -1, total = 0, offset = 0, window;

  // iterate over window-images within tube-sessoin; note last window index marked "annotated"
  for (std::vector<bool> *first, *i = first = m_exist_list[tube].begin(), *end = m_exist_list[tube].end(); i < end; i++)
    if ((i->size() > session) && (*i)[session])
    {
      total++;
      if (GetAnnotated(tube, window = i - first, session))
        last = window - offset;
    }
    else
      offset++;

    return last < 0? 0 : ((float)last + 1) / (float)total;
}*/


const std::vector<bool>* ProgressData::GetTubeList()
{
  return &m_tube_list;
}


const std::vector<bool>* ProgressData::GetWindowList()
{
  return &m_window_list;
}


const std::vector<bool>* ProgressData::GetSessionList()
{
  return &m_session_list;
}

void ProgressData::Reset()
{
  m_exist_count = m_annotated_count = m_done_count = m_tube_count = 0;
  m_exist_list.clear();
  m_annotated_list.clear();
  m_done_list.clear();
  m_tube_list.clear();
  m_window_list.clear();
  m_session_list.clear();
}

void ProgressData::ResetAnnotated()
{
  m_annotated_count = 0;
  m_annotated_list.clear();
}

void ProgressData::ResetDone()
{
  m_done_count = 0;
  m_done_list.clear();
}

int ProgressData::GetImageCount()
{
  return m_exist_count;
}

int ProgressData::GetAnnotatedCount()
{
  return m_annotated_count;
}

int ProgressData::GetDoneCount()
{
  return m_done_count;
}

int ProgressData::GetTubeCount()
{
  return m_tube_count;
}
