// TubeControl.cpp : implementation file
//

#include "stdafx.h"
#include "rootfly.h"
#include "TubeControl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// TubeControl

TubeControl::TubeControl(ProgressData *data)
{
  m_data = data;
  m_session = 1;
  
  COLORREF COLOR_NOT_EXIST = RGB(200,200,200);
  COLORREF COLOR_NOT_ANNOTATED = RGB(255,0,0);
  COLORREF COLOR_NOT_DONE = RGB(0,0,255);
  COLORREF COLOR_DONE = RGB(255,255,0);

  m_control_margin = m_tube_spacing = 20;
  m_tube_margin = m_window_spacing = 5;
  m_tube_top = 20;
  m_window_size = CSize(40, 30);

  int row_len = 3;
  int col_len = 4;
  m_tube_size = CSize(m_window_size.cx * row_len + (row_len - 1) * m_window_spacing + 2 * m_tube_margin,
    m_tube_top / 2 + m_window_size.cy * col_len + (col_len - 1) * m_window_spacing + 2 * m_tube_margin);
}

TubeControl::~TubeControl()
{
}

void TubeControl::OnFinalRelease()
{
	// When the last reference for an automation object is released
	// OnFinalRelease is called.  The base class will automatically
	// deletes the object.  Add additional cleanup required for your
	// object before calling the base class.

	CStatic::OnFinalRelease();
}


BEGIN_MESSAGE_MAP(TubeControl, CStatic)
	//{{AFX_MSG_MAP(TubeControl)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(TubeControl, CStatic)
	//{{AFX_DISPATCH_MAP(TubeControl)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_ITubeControl to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {D9B9BB98-A043-427E-A6F1-F755D2B1211B}
static const IID IID_ITubeControl =
{ 0xd9b9bb98, 0xa043, 0x427e, { 0xa6, 0xf1, 0xf7, 0x55, 0xd2, 0xb1, 0x21, 0x1b } };

BEGIN_INTERFACE_MAP(TubeControl, CStatic)
	INTERFACE_PART(TubeControl, IID_ITubeControl, Dispatch)
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// TubeControl message handlers

void TubeControl::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
  
  CRect rect;
  GetClientRect(rect);
  if (m_control_size != rect.Size())
  {
    m_control_size = rect.Size();
    PlaceWindows();
  }

  DrawWindows(&dc);
	
	// TODO: Add your message handler code here
	
	// Do not call CStatic::OnPaint() for painting messages
}

void TubeControl::SetSession(int session)
{
  m_session = session;
}

TubeControl::WindowRect::WindowRect() : CRect()
{
  window = 0;
}

TubeControl::WindowRect::WindowRect(CPoint point, CSize size, int window) : CRect(point, size)
{
  this->window = window;
}

TubeControl::TubeRect::TubeRect() : CRect()
{
  tube = 0;
  windows = std::vector<WindowRect>();
}

TubeControl::TubeRect::TubeRect(CPoint point, CSize size, int tube) : CRect(point, size)
{
  this->tube = tube;
  windows = std::vector<WindowRect>();
}

void TubeControl::PlaceWindows()
{
  int tube_x, tube_y, win_x, win_y, tube, win, t;

  const std::vector<bool> *tubes = m_data->GetTubeList();
  const std::vector<bool> *wins = m_data->GetWindowList();

  tube = t = -1;

  for (tube_y = m_control_margin; (tube + 1 < tubes->size()) && (tube_y + m_tube_size.cy + m_tube_top + m_control_margin <= m_control_size.cy); tube_y += m_tube_size.cy + m_tube_top + m_tube_spacing)
  {
    for (tube_x = m_control_margin; (tube + 1 < tubes->size()) && (tube_x + m_tube_size.cx + m_control_margin <= m_control_size.cx); tube_x += m_tube_size.cx + m_tube_spacing)
    {
      do 
      {
        tube++;
      } while ((tube < tubes->size()) && !(tubes->operator[](tube)));
      t++;
      
      if (tube >= tubes->size())
      {
        break;
      }
      
      if (m_tubes.size() <= t)
      {
        m_tubes.resize(t + 1, TubeRect());
      }
      m_tubes[t] = TubeRect(CPoint(tube_x, tube_y), m_tube_size, tube);
      if (m_tubes[t].windows.size() < wins->size())
      {
        m_tubes[t].windows.resize(wins->size(), WindowRect());
      }
      
      win = -1;

      for (win_y = m_tube_margin + m_tube_top; win_y + m_window_size.cy + m_tube_margin <= m_tube_size.cy + m_tube_top; win_y += m_window_size.cy + m_window_spacing)
      {
        for (win_x = m_tube_margin; win_x + m_window_size.cx + m_tube_margin <= m_tube_size.cx; win_x += m_window_size.cx + m_window_spacing)
        {
          do
          {
            win++;
          } while ((win < wins->size()) && !(wins->operator[](win)));

          if (win < wins->size())
          {
            m_tubes[t].windows[win] = WindowRect(CPoint(tube_x + win_x, tube_y + win_y), m_window_size, win);
          }
        }
      }
    }
  }
}

void TubeControl::DrawWindows(CPaintDC *dc)
{
  int t, w;

  for (t = 0; t < m_tubes.size(); t++)
  {
    if (m_tubes[t].Width() == 0)
    {
      continue;
    }

    DrawTube(m_tubes[t], dc);

    for (w = 0; w < m_tubes[t].windows.size(); w++)
    {
      if (m_tubes[t].windows[w].Width() == 0)
      {
        continue;
      }

      DrawWindow(m_tubes[t].windows[w], ChooseColor(m_tubes[t].tube, m_tubes[t].windows[w].window), dc);
    }
  }
}

void TubeControl::DrawTube(CRect tube, CPaintDC *dc)
{
	//draw the base of the tube
	dc->MoveTo(tube.left, tube.top + m_tube_top / 2);
	dc->LineTo(tube.left, tube.bottom + m_tube_top / 2);
	dc->LineTo(tube.right, tube.bottom + m_tube_top / 2);
	dc->LineTo(tube.right, tube.top + m_tube_top / 2);

	//draw the top of the tube
	CRect top(tube.left, tube.top, tube.right, tube.top + m_tube_top);
	dc->Ellipse(top);
}

void TubeControl::DrawWindow(CRect window, COLORREF color, CPaintDC *dc)
{
	//draw a rectangle
	dc->MoveTo(window.left, window.top);
	dc->LineTo(window.left, window.bottom);
	dc->LineTo(window.right, window.bottom);
	dc->LineTo(window.right, window.top);
	dc->LineTo(window.left, window.top);

  //color the rectangle
  dc->FillSolidRect(CRect(window.TopLeft() + CSize(1,1), window.Size() - CSize(1,1)), color);
}

COLORREF TubeControl::ChooseColor(int tube, int window)
{
  if (!m_data->GetExists(tube, window, m_session))
  {
    return RGB(200, 200, 200); //COLOR_NOT_EXIST;
  }
  else if (m_data->GetDone(tube, window, m_session))
  {
    return RGB(0, 255, 0); //COLOR_DONE;
  }
  else if (m_data->GetAnnotated(tube, window, m_session))
  {
    return RGB(255, 255, 0); //COLOR_NOT_DONE;
  }
  else
  {
    return RGB(255, 0, 0); //COLOR_NOT_ANNOTATED;
  }
}



void TubeControl::OnLButtonDown(UINT nFlags, CPoint point) 
{
  int tube, window;
  CRect rect;

  GetAt(point, &tube, &window, &rect);

  if ((tube != -1) && (window != -1))
  {
    m_data->SetDone(tube, window, m_session);
    RedrawWindow();
  }
	
	CStatic::OnLButtonDown(nFlags, point);
}

void TubeControl::GetAt(CPoint point, int *tube, int *window, CRect *rect)
{
  int t, w;

  for (t = 0; t < m_tubes.size(); t++)
  {
    if (!m_tubes[t].PtInRect(point))
    {
      continue;
    }

    for (w = 0; w < m_tubes[t].windows.size(); w++)
    {
      if (m_tubes[t].windows[w].PtInRect(point))
      {
        *tube = m_tubes[t].tube;
        *window = m_tubes[t].windows[w].window;
        *rect = m_tubes[t].windows[w];
        return;
      }
    }
  }

  *tube = *window = -1;
}
